# package information.
INFO = dict(
    name = "dungeon-generator",
    description = "Dungeon generator cli tool for rouge-like.",
    author = "Keisuke Ota",
    author_email = "arvelt.s@gmail.com",
    license = "MIT License",
    url = "https://github.com/arvelt/dungeon-generator",
    classifiers = [
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "License :: OSI Approved :: MIT License"
    ]
)
